@@ -48,6 +48,7 @@ from .jobs import (
     JobStatusHistory,
     JobTelemetry,
     Resources,
+    StdStream,
     Volume,
 )
 from .parser import Parser
@@ -70,6 +71,7 @@ __all__ = (
     "JobStatusHistory",
     "JobTelemetry",
     "Resources",
+    "StdStream",
     "Volume",
     "HTTPPort",
     "Users",
