@@ -1,31 +1,40 @@
 import asyncio
 import enum
 import json
-import shlex
-import signal
+import logging
 from contextlib import suppress
 from dataclasses import dataclass, field
 from datetime import datetime, timezone
-from typing import Any, AsyncIterator, Dict, Iterable, List, Mapping, Optional, Sequence
+from functools import partial
+from typing import (
+    Any,
+    AsyncIterator,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Union,
+)
 
+import aiohttp
 import attr
-import psutil
 from aiodocker.exceptions import DockerError
-from aiohttp import WSServerHandshakeError
+from aiohttp import WSMsgType, WSServerHandshakeError
 from dateutil.parser import isoparse
 from multidict import MultiDict
 from yarl import URL
 
-from neuromation.api.abc import (
+from .abc import (
     AbstractDockerImageProgress,
     ImageCommitFinished,
     ImageCommitStarted,
     ImageProgressPush,
     ImageProgressSave,
 )
-
 from .config import Config
-from .core import IllegalArgumentError, _Core
+from .core import _Core
 from .images import (
     _DummyProgress,
     _raise_on_error_chunk,
@@ -33,9 +42,12 @@ from .images import (
 )
 from .parser import Parser, Volume
 from .parsing_utils import LocalImage, RemoteImage, _as_repo_str, _is_in_neuro_registry
+from .url_utils import normalize_storage_path_uri
 from .utils import NoPublicConstructor, asynccontextmanager
 
 
+log = logging.getLogger(__name__)
+
 INVALID_IMAGE_NAME = "INVALID-IMAGE-NAME"
 
 
@@ -139,6 +151,47 @@ class JobTelemetry:
     gpu_memory: Optional[float] = None
 
 
+@dataclass(frozen=True)
+class ExecInspect:
+    id: str
+    running: bool
+    exit_code: int
+    job_id: str
+    tty: bool
+    entrypoint: str
+    command: str
+
+
+@dataclass(frozen=True)
+class Message:
+    fileno: int
+    data: bytes
+
+
+class StdStream:
+    def __init__(self, ws: aiohttp.ClientWebSocketResponse) -> None:
+        self._ws = ws
+        self._closing = False
+
+    async def close(self) -> None:
+        self._closing = True
+        await self._ws.close()
+
+    async def read_out(self) -> Optional[Message]:
+        if self._closing:
+            return None
+        msg = await self._ws.receive()
+        if msg.type in (WSMsgType.CLOSE, WSMsgType.CLOSING, WSMsgType.CLOSED):
+            self._closing = True
+            return None
+        return Message(msg.data[0], msg.data[1:])
+
+    async def write_in(self, data: bytes) -> None:
+        if self._closing:
+            return
+        await self._ws.send_bytes(data)
+
+
 class Jobs(metaclass=NoPublicConstructor):
     def __init__(self, core: _Core, config: Config, parse: Parser) -> None:
         self._core = core
@@ -159,7 +212,7 @@ class Jobs(metaclass=NoPublicConstructor):
     ) -> JobDescription:
         url = self._config.api_url / "jobs"
         payload: Dict[str, Any] = {
-            "container": _container_to_api(container),
+            "container": _container_to_api(container, self._config),
             "is_preemptible": is_preemptible,
         }
         if name:
@@ -322,123 +375,176 @@ class Jobs(metaclass=NoPublicConstructor):
                 if push_step:
                     progress.step(push_step)
 
-    async def exec(
-        self,
-        id: str,
-        cmd: Iterable[str],
-        *,
-        tty: bool = False,
-        no_key_check: bool = False,
-    ) -> int:
-        try:
-            job_status = await self.status(id)
-        except IllegalArgumentError as e:
-            raise ValueError(f"Job not found. Job Id = {id}") from e
-        if job_status.status != "running":
-            raise ValueError(f"Job is not running. Job Id = {job_status.id}")
-        payload = json.dumps(
-            {
-                "method": "job_exec",
-                "token": await self._config.token(),
-                "params": {"job": job_status.id, "command": list(cmd)},
-            }
-        )
-        command = ["ssh"]
-        if tty:
-            command += ["-tt"]
-        else:
-            command += ["-T"]
-        if no_key_check:  # pragma: no branch
-            command += [
-                "-o",
-                "StrictHostKeyChecking=no",
-                "-o",
-                "UserKnownHostsFile=/dev/null",
-            ]
-        server_url = job_status.ssh_server
-        port = server_url.port if server_url.port else 22
-        command += ["-p", str(port), f"{server_url.user}@{server_url.host}", payload]
-        proc = await asyncio.create_subprocess_exec(*command)
-        try:
-            return await proc.wait()
-        finally:
-            await _kill_proc_tree(proc.pid, timeout=10)
-            # add a sleep to get process watcher a chance to execute all callbacks
-            await asyncio.sleep(0.1)
-
     @asynccontextmanager
     async def port_forward(
         self, id: str, local_port: int, job_port: int, *, no_key_check: bool = False
     ) -> AsyncIterator[None]:
-        loop = asyncio.get_event_loop()
-        task = loop.create_task(
-            self._port_forward(id, local_port, job_port, no_key_check=no_key_check)
+        srv = await asyncio.start_server(
+            partial(self._port_forward, id=id, job_port=job_port),
+            "localhost",
+            local_port,
         )
-        yield
-        task.cancel()
-        with suppress(asyncio.CancelledError):
-            await task
+        try:
+            yield
+        finally:
+            srv.close()
+            await srv.wait_closed()
 
     async def _port_forward(
-        self, id: str, local_port: int, job_port: int, *, no_key_check: bool = False
+        self,
+        reader: asyncio.StreamReader,
+        writer: asyncio.StreamWriter,
+        id: str,
+        job_port: int,
     ) -> None:
         try:
-            job_status = await self.status(id)
-        except IllegalArgumentError as e:
-            raise ValueError(f"Job not found. Job Id = {id}") from e
-        if job_status.status != "running":
-            raise ValueError(f"Job is not running. Job Id = {job_status.id}")
-        payload = json.dumps(
-            {
-                "method": "job_port_forward",
-                "token": await self._config.token(),
-                "params": {"job": job_status.id, "port": job_port},
-            }
+            loop = asyncio.get_event_loop()
+            url = self._config.monitoring_url / id / "port_forward" / str(job_port)
+            auth = await self._config._api_auth()
+            ws = await self._core._session.ws_connect(
+                url,
+                headers={"Authorization": auth},
+                timeout=None,  # type: ignore
+                receive_timeout=None,
+                heartbeat=30,
+            )
+            tasks = []
+            tasks.append(loop.create_task(self._port_reader(ws, writer)))
+            tasks.append(loop.create_task(self._port_writer(ws, reader)))
+            try:
+                await asyncio.wait(tasks, return_when=asyncio.FIRST_COMPLETED)
+            finally:
+                for task in tasks:
+                    if not task.done():
+                        task.cancel()
+                        with suppress(asyncio.CancelledError):
+                            await task
+                writer.close()
+                await writer.wait_closed()
+                await ws.close()
+        except asyncio.CancelledError:
+            raise
+        except Exception:
+            log.exception("Unhandled exception during port-forwarding")
+
+    async def _port_reader(
+        self, ws: aiohttp.ClientWebSocketResponse, writer: asyncio.StreamWriter
+    ) -> None:
+        async for msg in ws:
+            assert msg.type == aiohttp.WSMsgType.BINARY
+            writer.write(msg.data)
+            await writer.drain()
+        writer.close()
+        await writer.wait_closed()
+
+    async def _port_writer(
+        self, ws: aiohttp.ClientWebSocketResponse, reader: asyncio.StreamReader
+    ) -> None:
+        while True:
+            data = await reader.read(4 * 1024 * 1024)
+            if not data:
+                # EOF
+                break
+            await ws.send_bytes(data)
+
+    @asynccontextmanager
+    async def attach(
+        self,
+        id: str,
+        *,
+        stdin: bool = False,
+        stdout: bool = False,
+        stderr: bool = False,
+        logs: bool = False,
+    ) -> AsyncIterator[StdStream]:
+        url = self._config.monitoring_url / id / "attach"
+        url = url.with_query(
+            stdin=str(int(stdin)),
+            stdout=str(int(stdout)),
+            stderr=str(int(stderr)),
+            logs=str(int(logs)),
         )
-        proxy_command = ["ssh"]
-        if no_key_check:  # pragma: no branch
-            proxy_command += [
-                "-o",
-                "StrictHostKeyChecking=no",
-                "-o",
-                "UserKnownHostsFile=/dev/null",
-            ]
-        server_url = job_status.ssh_server
-        port = server_url.port if server_url.port else 22
-        proxy_command += [
-            "-p",
-            str(port),
-            f"{server_url.user}@{server_url.host}",
-            payload,
-        ]
-        proxy_command_str = " ".join(shlex.quote(s) for s in proxy_command)
-        command = [
-            "ssh",
-            "-NL",
-            f"{local_port}:{job_status.internal_hostname}:{job_port}",
-            "-o",
-            f"ProxyCommand={proxy_command_str}",
-            "-o",
-            "ExitOnForwardFailure=yes",
-        ]
-        if no_key_check:  # pragma: no branch
-            command += [
-                "-o",
-                "StrictHostKeyChecking=no",
-                "-o",
-                "UserKnownHostsFile=/dev/null",
-            ]
-        command += [f"{server_url.user}@{server_url.host}"]
-        proc = await asyncio.create_subprocess_exec(*command)
+        auth = await self._config._api_auth()
+        ws = await self._core._session.ws_connect(
+            url,
+            headers={"Authorization": auth},
+            timeout=None,  # type: ignore
+            receive_timeout=None,
+            heartbeat=30,
+        )
+
         try:
-            result = await proc.wait()
-            if result != 0:
-                raise ValueError(f"error code {result}")
-            return
+            yield StdStream(ws)
         finally:
-            await _kill_proc_tree(proc.pid, timeout=10)
-            # add a sleep to get process watcher a chance to execute all callbacks
-            await asyncio.sleep(0.1)
+            await ws.close()
+
+    async def resize(self, id: str, *, w: int, h: int) -> None:
+        url = self._config.monitoring_url / id / "resize"
+        url = url.with_query(w=w, h=h)
+        auth = await self._config._api_auth()
+        async with self._core.request("POST", url, auth=auth):
+            pass
+
+    async def exec_create(self, id: str, cmd: str, *, tty: bool = False) -> str:
+        payload = {
+            "command": cmd,
+            "stdin": True,
+            "stdout": True,
+            "stderr": True,
+            "tty": tty,
+        }
+        url = self._config.monitoring_url / id / "exec_create"
+        auth = await self._config._api_auth()
+        async with self._core.request("POST", url, json=payload, auth=auth) as resp:
+            ret = await resp.json()
+            return ret["exec_id"]
+
+    async def exec_resize(self, id: str, exec_id: str, *, w: int, h: int) -> None:
+        url = self._config.monitoring_url / id / exec_id / "exec_resize"
+        url = url.with_query(w=w, h=h)
+        auth = await self._config._api_auth()
+        async with self._core.request("POST", url, auth=auth) as resp:
+            resp
+
+    async def exec_inspect(self, id: str, exec_id: str) -> ExecInspect:
+        url = self._config.monitoring_url / id / exec_id / "exec_inspect"
+        auth = await self._config._api_auth()
+        async with self._core.request("GET", url, auth=auth) as resp:
+            data = await resp.json()
+            return ExecInspect(
+                id=data["id"],
+                running=data["running"],
+                exit_code=data["exit_code"],
+                job_id=data["job_id"],
+                tty=data["tty"],
+                entrypoint=data["entrypoint"],
+                command=data["command"],
+            )
+
+    @asynccontextmanager
+    async def exec_start(self, id: str, exec_id: str) -> AsyncIterator[StdStream]:
+        url = self._config.monitoring_url / id / exec_id / "exec_start"
+        auth = await self._config._api_auth()
+
+        ws = await self._core._session.ws_connect(
+            url,
+            headers={"Authorization": auth},
+            timeout=None,  # type: ignore
+            receive_timeout=None,
+            heartbeat=30,
+        )
+
+        try:
+            yield StdStream(ws)
+        finally:
+            await ws.close()
+
+    async def send_signal(self, id: str, signal: Union[str, int]) -> None:
+        url = self._config.monitoring_url / id / "kill"
+        url = url.with_query(signal=signal)
+        auth = await self._config._api_auth()
+        async with self._core.request("POST", url, auth=auth) as resp:
+            resp
 
 
 #  ############## Internal helpers ###################
@@ -544,7 +650,7 @@ def _container_from_api(data: Dict[str, Any], parse: Parser) -> Container:
     )
 
 
-def _container_to_api(container: Container) -> Dict[str, Any]:
+def _container_to_api(container: Container, config: Config) -> Dict[str, Any]:
     primitive: Dict[str, Any] = {
         "image": _as_repo_str(container.image),
         "resources": _resources_to_api(container.resources),
@@ -558,7 +664,7 @@ def _container_to_api(container: Container) -> Dict[str, Any]:
     if container.env:
         primitive["env"] = container.env
     if container.volumes:
-        primitive["volumes"] = [_volume_to_api(v) for v in container.volumes]
+        primitive["volumes"] = [_volume_to_api(v, config) for v in container.volumes]
     if container.tty:
         primitive["tty"] = True
     return primitive
@@ -619,9 +725,12 @@ def _job_telemetry_from_api(value: Dict[str, Any]) -> JobTelemetry:
     )
 
 
-def _volume_to_api(volume: Volume) -> Dict[str, Any]:
+def _volume_to_api(volume: Volume, config: Config) -> Dict[str, Any]:
+    uri = normalize_storage_path_uri(
+        volume.storage_uri, config.username, config.cluster_name
+    )
     resp: Dict[str, Any] = {
-        "src_storage_uri": str(volume.storage_uri),
+        "src_storage_uri": str(uri),
         "dst_path": volume.container_path,
         "read_only": bool(volume.read_only),
     }
@@ -641,50 +750,3 @@ def _parse_datetime(dt: Optional[str]) -> Optional[datetime]:
     if dt is None:
         return None
     return isoparse(dt)
-
-
-async def _kill_proc_tree(
-    pid: int,
-    sig: int = signal.SIGTERM,
-    include_parent: bool = True,
-    timeout: int = None,
-) -> None:
-    """Kill a process tree (including grandchildren) with signal
-    "sig".
-    """
-
-    def inner() -> None:
-        try:
-            parent = psutil.Process(pid)
-            children = parent.children(recursive=True)
-            zombies: List[psutil.Process] = []
-            # Try to kill all children first
-            for p in children:
-                try:
-                    p.send_signal(sig)
-                except psutil.NoSuchProcess:
-                    pass
-            _, children_alive = psutil.wait_procs(children, timeout=timeout)
-            # then kill parent
-            if include_parent:
-                parent.send_signal(sig)
-                try:
-                    parent.wait(timeout=timeout)
-                    # and try to kill again left childrent
-                    _, children_alive = psutil.wait_procs(
-                        children_alive, timeout=timeout
-                    )
-                    zombies.extend(children_alive)
-                except psutil.TimeoutExpired:
-                    zombies.append(parent)
-            else:
-                zombies.extend(children_alive)
-
-            if zombies:
-                raise RuntimeWarning(f"Possible zombie subprocesses: {zombies}")
-
-        except psutil.NoSuchProcess:
-            pass
-
-    loop = asyncio.get_event_loop()
-    await loop.run_in_executor(None, inner)
