@@ -13,6 +13,7 @@ from email.utils import parsedate
 from pathlib import Path
 from stat import S_ISREG
 from typing import (
+    AbstractSet,
     Any,
     AsyncIterator,
     Awaitable,
@@ -20,7 +21,9 @@ from typing import (
     Dict,
     Iterable,
     Optional,
+    Sequence,
     Tuple,
+    Union,
     cast,
 )
 
@@ -40,6 +43,7 @@ from .abc import (
 )
 from .config import Config
 from .core import ResourceNotFound, _Core
+from .file_filter import FileFilter
 from .url_utils import (
     _extract_path,
     normalize_local_path_uri,
@@ -399,6 +403,7 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool = False,
         filter: Optional[Callable[[str], Awaitable[bool]]] = None,
+        ignore_file_names: Union[Sequence[str], AbstractSet[str]] = (),
         progress: Optional[AbstractRecursiveFileProgress] = None,
     ) -> None:
         if filter is None:
@@ -416,7 +421,14 @@ class Storage(metaclass=NoPublicConstructor):
         await run_progress(
             queued,
             self._upload_dir(
-                src, path, dst, "", update=update, filter=filter, progress=queued,
+                src,
+                path,
+                dst,
+                "",
+                update=update,
+                filter=filter,
+                ignore_file_names=ignore_file_names,
+                progress=queued,
             ),
         )
 
@@ -429,6 +441,7 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool,
         filter: Callable[[str], Awaitable[bool]],
+        ignore_file_names: Union[Sequence[str], AbstractSet[str]],
         progress: "QueuedProgress",
     ) -> None:
         tasks = []
@@ -452,13 +465,25 @@ class Storage(metaclass=NoPublicConstructor):
                         await self.mkdir(dst, exist_ok=True)
         except FileExistsError:
             raise NotADirectoryError(errno.ENOTDIR, "Not a directory", str(dst))
+
         await progress.enter(StorageProgressEnterDir(src, dst))
         loop = asyncio.get_event_loop()
         async with self._file_sem:
             folder = await loop.run_in_executor(None, lambda: list(src_path.iterdir()))
+
+        if ignore_file_names:
+            for child in folder:
+                if child.name in ignore_file_names and child.is_file():
+                    log.debug(f"Load ignore file {rel_path}{child.name}")
+                    file_filter = FileFilter(filter)
+                    file_filter.read_from_file(child, prefix=rel_path)
+                    filter = file_filter.match
+
         for child in folder:
             name = child.name
-            child_rel_path = f"{rel_path}/{name}" if rel_path else name
+            child_rel_path = f"{rel_path}{name}"
+            if child.is_dir():
+                child_rel_path += "/"
             if not await filter(child_rel_path):
                 log.debug(f"Skip {child_rel_path}")
                 continue
@@ -481,6 +506,7 @@ class Storage(metaclass=NoPublicConstructor):
                         child_rel_path,
                         update=update,
                         filter=filter,
+                        ignore_file_names=ignore_file_names,
                         progress=progress,
                     )
                 )
@@ -607,7 +633,9 @@ class Storage(metaclass=NoPublicConstructor):
 
         for child in folder:
             name = child.name
-            child_rel_path = f"{rel_path}/{name}" if rel_path else name
+            child_rel_path = f"{rel_path}{name}"
+            if child.is_dir():
+                child_rel_path += "/"
             if not await filter(child_rel_path):
                 log.debug(f"Skip {child_rel_path}")
                 continue
