@@ -7,7 +7,7 @@ from dataclasses import dataclass
 from typing import Iterable, Iterator, List
 
 import humanize
-from click import style, unstyle
+from click import secho, style, unstyle
 
 from neuromation.api import (
     JobDescription,
@@ -33,6 +33,16 @@ COLORS = {
 }
 
 
+if sys.platform == "win32":
+    SPINNER = itertools.cycle(r"-\|/")
+else:
+    SPINNER = itertools.cycle("◢◣◤◥")
+
+
+def bold(text: str) -> str:
+    return style(text, bold=True)
+
+
 def format_job_status(status: JobStatus) -> str:
     return style(status.value, fg=COLORS.get(status, "reset"))
 
@@ -57,139 +67,97 @@ def format_timedelta(delta: datetime.timedelta) -> str:
     )
 
 
-class JobFormatter:
-    def __init__(self, quiet: bool = True) -> None:
-        self._quiet = quiet
-
-    def __call__(self, job: JobDescription) -> str:
-        job_id = job.id
-        if self._quiet:
-            return job_id
-        out = []
-        out.append(
-            style("Job ID", bold=True)
-            + f": {job_id} "
-            + style("Status", bold=True)
-            + f": {format_job_status(job.status)}"
-        )
-        if job.name:
-            out.append(style("Name", bold=True) + f": {job.name}")
-            job_alias = job.name
-        else:
-            job_alias = job.id
-        http_url = job.http_url
-        if http_url:
-            out.append(style("Http URL", bold=True) + f": {http_url}")
-        out.append(style("Shortcuts", bold=True) + ":")
-
-        out.append(
-            f"  neuro status {job_alias}     " + style("# check job status", dim=True)
-        )
-        out.append(
-            f"  neuro logs {job_alias}       " + style("# monitor job stdout", dim=True)
-        )
-        out.append(
-            f"  neuro top {job_alias}        "
-            + style("# display real-time job telemetry", dim=True)
-        )
-        out.append(
-            f"  neuro exec {job_alias} bash  "
-            + style("# execute bash shell to the job", dim=True)
-        )
-        out.append(f"  neuro kill {job_alias}       " + style("# kill job", dim=True))
-        return "\n".join(out)
-
-
 class JobStatusFormatter:
     def __init__(self, uri_formatter: URIFormatter) -> None:
         self._format_uri = uri_formatter
         self._format_image = image_formatter(uri_formatter=uri_formatter)
 
     def __call__(self, job_status: JobDescription) -> str:
-        result: str = f"Job: {job_status.id}\n"
+        assert job_status.history is not None
+        lines = []
+
+        def add(descr: str, value: str) -> None:
+            lines.append(f"{bold(descr)}: {value}")
+
+        add("Job", job_status.id)
         if job_status.name:
-            result += f"Name: {job_status.name}\n"
+            add("Name", job_status.name)
         if job_status.tags:
-            result += f"Tags: {', '.join(job_status.tags)}\n"
-        result += f"Owner: {job_status.owner if job_status.owner else ''}\n"
-        result += f"Cluster: {job_status.cluster_name}\n"
+            add("Tags", ", ".join(job_status.tags))
+        add("Owner", job_status.owner or "")
+        add("Cluster", job_status.cluster_name)
         if job_status.description:
-            result += f"Description: {job_status.description}\n"
-        result += f"Status: {job_status.status}"
-        if (
-            job_status.history
-            and job_status.history.reason
-            and job_status.status in [JobStatus.FAILED, JobStatus.PENDING]
-        ):
-            result += f" ({job_status.history.reason})"
-        result += f"\nImage: {self._format_image(job_status.container.image)}\n"
+            add("Description", job_status.description)
+        status_str = format_job_status(job_status.status)
+        if job_status.history.reason and job_status.status in [
+            JobStatus.FAILED,
+            JobStatus.PENDING,
+        ]:
+            status_str += f" ({job_status.history.reason})"
+        add("Status", status_str)
+        add("Image", self._format_image(job_status.container.image))
 
         if job_status.container.entrypoint:
-            result += f"Entrypoint: {job_status.container.entrypoint}\n"
-        result += f"Command: {job_status.container.command}\n"
+            add("Entrypoint", job_status.container.entrypoint)
+        if job_status.container.command:
+            add("Command", job_status.container.command)
         resource_formatter = ResourcesFormatter()
-        result += resource_formatter(job_status.container.resources) + "\n"
-        result += f"Preemptible: {job_status.is_preemptible}\n"
+        lines.append(resource_formatter(job_status.container.resources))
+        if job_status.is_preemptible:
+            add("Preemptible", "True")
         if job_status.restart_policy != JobRestartPolicy.NEVER:
-            result += f"Restart policy: {job_status.restart_policy}\n"
+            add("Restart policy", job_status.restart_policy)
         if job_status.life_span is not None:
             limit = (
                 "no limit"
                 if job_status.life_span == 0
                 else format_timedelta(datetime.timedelta(seconds=job_status.life_span))
             )
-            result += f"Life span: {limit}\n"
+            add("Life span", limit)
 
-        if job_status.container.tty:
-            result += "TTY: True\n"
+        add("TTY", str(job_status.container.tty))
 
         if job_status.container.volumes:
             rows = [
                 (
                     volume.container_path,
-                    f"{self._format_uri(volume.storage_uri)}",
+                    self._format_uri(volume.storage_uri),
                     "READONLY" if volume.read_only else " ",
                 )
                 for volume in job_status.container.volumes
             ]
-            result += "Volumes:" + "\n  "
-            result += "\n  ".join(table(rows)) + "\n"
+            lines.append(f"{bold('Volumes')}:")
+            lines.extend(f"  {i}" for i in table(rows))
 
         if job_status.internal_hostname:
-            result += f"Internal Hostname: {job_status.internal_hostname}\n"
+            add("Internal Hostname", job_status.internal_hostname)
         if job_status.http_url:
-            result = f"{result}Http URL: {job_status.http_url}\n"
+            add("Http URL", str(job_status.http_url))
         if job_status.container.http:
-            result = (
-                f"{result}Http authentication: "
-                f"{job_status.container.http.requires_auth}\n"
-            )
+            add("Http authentication", str(job_status.container.http.requires_auth))
         if job_status.container.env:
-            result += f"Environment:\n"
+            lines.append(f"{bold('Environment')}:")
             for key, value in job_status.container.env.items():
-                result += f"{key}={value}\n"
+                lines.append(f"  {key}={value}")
 
-        assert job_status.history is not None
         assert job_status.history.created_at is not None
-        created_at = job_status.history.created_at.isoformat()
-        result = f"{result}Created: {created_at}"
+        add("Created", job_status.history.created_at.isoformat())
         if job_status.status in [
             JobStatus.RUNNING,
             JobStatus.FAILED,
             JobStatus.SUCCEEDED,
         ]:
             assert job_status.history.started_at is not None
-            started_at = job_status.history.started_at.isoformat()
-            result += "\n" f"Started: {started_at}"
+            add("Started", job_status.history.started_at.isoformat())
         if job_status.status in [JobStatus.FAILED, JobStatus.SUCCEEDED]:
             assert job_status.history.finished_at is not None
-            finished_at = job_status.history.finished_at.isoformat()
-            result += "\n" f"Finished: {finished_at}"
-            result += "\n" f"Exit code: {job_status.history.exit_code}"
-        if job_status.status == JobStatus.FAILED:
-            result += "\n===Description===\n"
-            result += f"{job_status.history.description}\n================="
-        return result
+            add("Finished", job_status.history.finished_at.isoformat())
+            add("Exit code", str(job_status.history.exit_code))
+        if job_status.status == JobStatus.FAILED and job_status.history.description:
+            lines.append(bold("===Description==="))
+            lines.append(job_status.history.description)
+            lines.append("=================")
+        return "\n".join(lines)
 
 
 class JobTelemetryFormatter:
@@ -328,27 +296,33 @@ class TabularJobsFormatter(BaseJobsFormatter):
 
 class ResourcesFormatter:
     def __call__(self, resources: Resources) -> str:
-        lines = list()
-        lines.append("Memory: " + format_size(resources.memory_mb * 1024 ** 2))
-        lines.append(f"CPU: {resources.cpu:0.1f}")
+        lines = []
+
+        def add(descr: str, value: str) -> None:
+            lines.append(f"{bold(descr)}: {value}")
+
+        add("Memory", format_size(resources.memory_mb * 1024 ** 2))
+        add("CPU", f"{resources.cpu:0.1f}")
         if resources.gpu:
-            lines.append(f"GPU: {resources.gpu:0.1f} x {resources.gpu_model}")
+            add("GPU", f"{resources.gpu:0.1f} x {resources.gpu_model}")
 
         if resources.tpu_type:
-            lines.append(f"TPU: {resources.tpu_type}/{resources.tpu_software_version}")
+            add("TPU", f"{resources.tpu_type}/{resources.tpu_software_version}")
 
-        additional = list()
+        additional = []
         if resources.shm:
             additional.append("Extended SHM space")
 
         if additional:
-            lines.append(f'Additional: {",".join(additional)}')
+            add("Additional", ",".join(additional))
 
         indent = "  "
-        return "Resources:\n" + indent + f"\n{indent}".join(lines)
+        return f"{bold('Resources')}:\n" + indent + f"\n{indent}".join(lines)
 
 
 class JobStartProgress:
+    time_factory = staticmethod(time.monotonic)
+
     @classmethod
     def create(cls, tty: bool, color: bool, quiet: bool) -> "JobStartProgress":
         if quiet:
@@ -357,10 +331,14 @@ class JobStartProgress:
             return DetailedJobStartProgress(color)
         return StreamJobStartProgress()
 
-    def __call__(self, job: JobDescription) -> None:
+    def begin(self, job: JobDescription) -> None:
+        # Quiet mode
+        print(job.id)
+
+    def step(self, job: JobDescription) -> None:
         pass
 
-    def close(self) -> None:
+    def end(self, job: JobDescription) -> None:
         pass
 
     def _get_status_reason_message(self, job: JobDescription) -> str:
@@ -379,18 +357,24 @@ class JobStartProgress:
 
 class DetailedJobStartProgress(JobStartProgress):
     def __init__(self, color: bool):
-        self._time = time.time()
+        self._time = self.time_factory()
         self._color = color
         self._prev = ""
-        if sys.platform == "win32":
-            self._spinner = itertools.cycle("-\\|/")
-        else:
-            self._spinner = itertools.cycle("◢◣◤◥")
+        self._spinner = SPINNER
         self._printer = TTYPrinter()
         self._lineno = 0
 
-    def __call__(self, job: JobDescription) -> None:
-        new_time = time.time()
+    def begin(self, job: JobDescription) -> None:
+        self._printer.print(
+            style("√ ", fg="green") + style("Job ID", bold=True) + f": {job.id} "
+        )
+        if job.name:
+            self._printer.print(
+                style("√ ", fg="green") + style("Name", bold=True) + f": {job.name}"
+            )
+
+    def step(self, job: JobDescription) -> None:
+        new_time = self.time_factory()
         dt = new_time - self._time
         msg = "Status: " + format_job_status(job.status)
         reason = self._get_status_reason_message(job)
@@ -400,6 +384,14 @@ class DetailedJobStartProgress(JobStartProgress):
         if description:
             msg += " " + description
 
+        if job.status == JobStatus.PENDING:
+            msg = style("- ", fg="yellow") + msg
+        elif job.status == JobStatus.FAILED:
+            msg = style("× ", fg="red") + msg
+        else:
+            # RUNNING or SUCCEDED
+            msg = style("√ ", fg="green") + msg
+
         if not self._color:
             msg = unstyle(msg)
         if msg != self._prev:
@@ -413,13 +405,38 @@ class DetailedJobStartProgress(JobStartProgress):
                 f"{msg} {next(self._spinner)} [{dt:.1f} sec]", lineno=self._lineno
             )
 
+    def end(self, job: JobDescription) -> None:
+        out = []
+
+        if job.status != JobStatus.FAILED:
+            http_url = job.http_url
+            if http_url:
+                out.append(
+                    style("√ ", fg="green")
+                    + style("Http URL", bold=True)
+                    + f": {http_url}"
+                )
+            if job.life_span:
+                limit = humanize.naturaldelta(datetime.timedelta(seconds=job.life_span))
+                out.append(
+                    style("√ ", fg="green")
+                    + style(f"The job will die in {limit}. ", fg="yellow",)
+                    + "See --life-span option documentation for details.",
+                )
+            self._printer.print("\n".join(out))
+
 
 class StreamJobStartProgress(JobStartProgress):
     def __init__(self) -> None:
         self._printer = StreamPrinter()
         self._prev = ""
 
-    def __call__(self, job: JobDescription) -> None:
+    def begin(self, job: JobDescription) -> None:
+        self._printer.print(f"Job ID: {job.id}")
+        if job.name:
+            self._printer.print(f"Name: {job.name}")
+
+    def step(self, job: JobDescription) -> None:
         msg = f"Status: {job.status}"
         reason = self._get_status_reason_message(job)
         if reason:
@@ -428,8 +445,218 @@ class StreamJobStartProgress(JobStartProgress):
         if description:
             msg += " " + description
 
+        if job.status != JobStatus.PENDING:
+            msg += "\n"
+
         if msg != self._prev:
             self._printer.print(msg)
             self._prev = msg
         else:
             self._printer.tick()
+
+    def end(self, job: JobDescription) -> None:
+        pass
+
+
+class JobStopProgress:
+    TIMEOUT = 15 * 60
+    time_factory = staticmethod(time.monotonic)
+
+    @classmethod
+    def create(cls, tty: bool, color: bool, quiet: bool) -> "JobStopProgress":
+        if quiet:
+            return JobStopProgress()
+        elif tty:
+            return DetailedJobStopProgress(color)
+        return StreamJobStopProgress()
+
+    def __init__(self) -> None:
+        self._time = self.time_factory()
+
+    def kill(self, job: JobDescription) -> None:
+        pass
+
+    def detach(self, job: JobDescription) -> None:
+        pass
+
+    def step(self, job: JobDescription) -> bool:
+        # return False if timeout, True otherwise
+        new_time = self.time_factory()
+        if new_time - self._time > self.TIMEOUT:
+            self.timeout(job)
+            return False
+        else:
+            self.tick(job)
+            return True
+
+    def tick(self, job: JobDescription) -> None:
+        pass
+
+    def timeout(self, job: JobDescription) -> None:
+        pass
+
+
+class DetailedJobStopProgress(JobStopProgress):
+    def __init__(self, color: bool):
+        super().__init__()
+        self._color = color
+        self._spinner = SPINNER
+        self._printer = TTYPrinter()
+        self._lineno = 0
+
+    def detach(self, job: JobDescription) -> None:
+        secho()
+        secho("× Terminal was detached but job is still running", fg="red")
+        secho("Re-attach to job:", dim=True, fg="yellow")
+        secho(f"  neuro attach {job.id}", dim=True)
+        secho("Check job status:", dim=True, fg="yellow")
+        secho(f"  neuro status {job.id}", dim=True)
+        secho("Kill job:", dim=True, fg="yellow")
+        secho(f"  neuro kill {job.id}", dim=True)
+        secho("Fetch job logs:", dim=True, fg="yellow")
+        secho(f"  neuro logs {job.id}", dim=True)
+
+    def kill(self, job: JobDescription) -> None:
+        secho()
+        secho("× Job was killed", fg="red")
+        secho("Get job status:", dim=True, fg="yellow")
+        secho(f"  neuro status {job.id}", dim=True)
+        secho("Fetch job logs:", dim=True, fg="yellow")
+        secho(f"  neuro logs {job.id}", dim=True)
+
+    def tick(self, job: JobDescription) -> None:
+        new_time = self.time_factory()
+        dt = new_time - self._time
+
+        if job.status == JobStatus.RUNNING:
+            msg = (
+                style("-", fg="yellow")
+                + f" Wait for stop {next(self._spinner)} [{dt:.1f} sec]"
+            )
+        else:
+            msg = style("√", fg="green") + " Stopped"
+
+        if not self._color:
+            msg = unstyle(msg)
+        self._printer.print(
+            msg, lineno=self._lineno,
+        )
+
+    def timeout(self, job: JobDescription) -> None:
+        secho()
+        secho("× Warning !!!", fg="red")
+        secho(
+            "× The attached session was disconnected but the job is still alive.",
+            fg="red",
+        )
+        secho("Reconnect to the job:", dim=True, fg="yellow")
+        secho(f"  neuro attach {job.id}", dim=True)
+        secho("Terminate the job:", dim=True, fg="yellow")
+        secho(f"  neuro kill {job.id}", dim=True)
+
+
+class StreamJobStopProgress(JobStopProgress):
+    def __init__(self) -> None:
+        super().__init__()
+        self._printer = StreamPrinter()
+        self._printer.print("Wait for stopping")
+
+    def detach(self, job: JobDescription) -> None:
+        pass
+
+    def kill(self, job: JobDescription) -> None:
+        self._printer.print("Job was killed")
+
+    def tick(self, job: JobDescription) -> None:
+        self._printer.tick()
+
+    def timeout(self, job: JobDescription) -> None:
+        self._printer.print("")
+        self._printer.print("Warning !!!")
+        self._printer.print(
+            "The attached session was disconnected but the job is still alive."
+        )
+
+
+class ExecStopProgress:
+    TIMEOUT = 15
+    time_factory = staticmethod(time.monotonic)
+
+    @classmethod
+    def create(cls, tty: bool, color: bool, quiet: bool) -> "ExecStopProgress":
+        if quiet:
+            return ExecStopProgress()
+        elif tty:
+            return DetailedExecStopProgress(color)
+        return StreamExecStopProgress()
+
+    def __init__(self) -> None:
+        self._time = self.time_factory()
+
+    def __call__(self, running: bool) -> bool:
+        # return False if timeout, True otherwise
+        new_time = self.time_factory()
+        if new_time - self._time > self.TIMEOUT:
+            self.timeout()
+            return False
+        else:
+            self.tick(running)
+            return True
+
+    def tick(self, running: bool) -> None:
+        pass
+
+    def timeout(self) -> None:
+        pass
+
+
+class DetailedExecStopProgress(ExecStopProgress):
+    def __init__(self, color: bool):
+        super().__init__()
+        self._color = color
+        self._spinner = SPINNER
+        self._printer = TTYPrinter()
+        self._lineno = 0
+
+    def tick(self, running: bool) -> None:
+        new_time = self.time_factory()
+        dt = new_time - self._time
+
+        if running:
+            msg = (
+                style("-", fg="yellow")
+                + f"Wait for stopping {next(self._spinner)} [{dt:.1f} sec]"
+            )
+        else:
+            msg = style("√", fg="green") + " Stopped"
+
+        self._printer.print(
+            msg, lineno=self._lineno,
+        )
+
+    def timeout(self) -> None:
+        secho()
+        secho("× Warning !!!", fg="red")
+        secho(
+            "× The attached session was disconnected "
+            "but the exec process is still alive.",
+            fg="red",
+        )
+
+
+class StreamExecStopProgress(ExecStopProgress):
+    def __init__(self) -> None:
+        super().__init__()
+        self._printer = StreamPrinter()
+        self._printer.print("Wait for stopping")
+
+    def tick(self, running: bool) -> None:
+        self._printer.tick()
+
+    def timeout(self) -> None:
+        print()
+        print("Warning !!!")
+        print(
+            "The attached session was disconnected "
+            "but the exec process is still alive."
+        )
