@@ -5,7 +5,6 @@ import os
 import re
 import shlex
 import sys
-import textwrap
 import uuid
 import webbrowser
 from datetime import datetime, timedelta
@@ -28,13 +27,14 @@ from neuromation.api import (
     Resources,
     Volume,
 )
-from neuromation.cli.formatters import DockerImageProgress
+from neuromation.cli.formatters.images import DockerImageProgress
 from neuromation.cli.formatters.utils import (
     URIFormatter,
     image_formatter,
     uri_formatter,
 )
 
+from .ael import process_attach, process_exec, process_logs
 from .click_types import (
     JOB,
     JOB_COLUMNS,
@@ -54,7 +54,6 @@ from .defaults import (
 )
 from .formatters.jobs import (
     BaseJobsFormatter,
-    JobFormatter,
     JobStartProgress,
     JobStatusFormatter,
     JobTelemetryFormatter,
@@ -93,6 +92,19 @@ REGEX_JOB_LIFE_SPAN = re.compile(
 )
 
 
+TTY_OPT = option(
+    "-t/-T",
+    "--tty/--no-tty",
+    is_flag=True,
+    default=None,
+    help=(
+        "Allocate a TTY, can be useful for interactive jobs. "
+        "By default is on if the command is executed from a terminal, "
+        "non-tty mode is used if executed from a script."
+    ),
+)
+
+
 def _get_neuro_mountpoint(username: str) -> str:
     return f"{ROOT_MOUNTPOINT}/{username}"
 
@@ -187,9 +199,7 @@ def job() -> None:
     show_default=True,
     help="Request extended '/dev/shm' space",
 )
-@option(
-    "--http", type=int, metavar="PORT", help="Enable HTTP port forwarding to container"
-)
+@option("--http", type=int, metavar="PORT", help="Enable HTTP port forwarding to a job")
 @option(
     "--http-auth/--no-http-auth",
     is_flag=True,
@@ -302,7 +312,7 @@ def job() -> None:
     is_flag=True,
     help="Don't attach to job logs and don't wait for exit code",
 )
-@option("-t", "--tty", is_flag=True, help="Allocate a TTY")
+@TTY_OPT
 async def submit(
     root: Root,
     image: RemoteImage,
@@ -330,7 +340,7 @@ async def submit(
     pass_config: bool,
     browse: bool,
     detach: bool,
-    tty: bool,
+    tty: Optional[bool],
 ) -> None:
     """
     Submit an image to run on the cluster.
@@ -351,6 +361,8 @@ async def submit(
     # registry, run /script.sh and pass arg1 arg2 arg3 as its arguments:
     neuro submit image:my-ubuntu:latest --entrypoint=/script.sh arg1 arg2 arg3
     """
+    if tty is None:
+        tty = root.tty
     await run_job(
         root,
         image=image,
@@ -385,23 +397,7 @@ async def submit(
 @command(context_settings=dict(allow_interspersed_args=False))
 @argument("job", type=JOB)
 @argument("cmd", nargs=-1, type=click.UNPROCESSED, required=True)
-@option(
-    "-t/-T",
-    "--tty/--no-tty",
-    default=True,
-    is_flag=True,
-    help="Allocate virtual tty. Useful for interactive jobs.",
-)
-@option(
-    "-i/-I",
-    "--interactive/--no-interactive",
-    default=None,
-    is_flag=True,
-    help=(
-        "Keep STDIN open even if not attached. "
-        "On for tty by default, false otherwise."
-    ),
-)
+@TTY_OPT
 @option(
     "--no-key-check",
     is_flag=True,
@@ -413,13 +409,13 @@ async def submit(
     default=0,
     type=float,
     show_default=True,
+    hidden=True,
     help="Maximum allowed time for executing the command, 0 for no timeout",
 )
 async def exec(
     root: Root,
     job: str,
-    tty: bool,
-    interactive: Optional[bool],
+    tty: Optional[bool],
     no_key_check: bool,
     cmd: Sequence[str],
     timeout: float,
@@ -436,16 +432,13 @@ async def exec(
     neuro exec --no-tty my-job ls -l
     """
     real_cmd = _parse_cmd(cmd)
-    id = await resolve_job(
+    job = await resolve_job(
         job, client=root.client, status={JobStatus.PENDING, JobStatus.RUNNING}
     )
-    if interactive is None:
-        interactive = tty
-    async with async_timeout.timeout(timeout if timeout else None,):
-        retcode = await root.client.jobs.exec(
-            id, shlex.split(real_cmd), tty=tty, no_key_check=no_key_check,
-        )
-        sys.exit(retcode)
+    if tty is None:
+        tty = root.tty
+    _check_tty(root, tty)
+    await process_exec(root, job, real_cmd, tty)
 
 
 @command()
@@ -475,7 +468,7 @@ async def port_forward(
     rsync -avxzhe "ssh -p 2222" root@localhost:/data .
 
     # Forward few ports at once
-    neuro job port-forward my-job- 2080:80 2222:22 2000:100
+    neuro job port-forward my-job 2080:80 2222:22 2000:100
 
     """
     job_id = await resolve_job(
@@ -505,7 +498,7 @@ async def port_forward(
 @argument("job", type=JOB)
 async def logs(root: Root, job: str) -> None:
     """
-    Print the logs for a container.
+    Print the logs for a job.
     """
     id = await resolve_job(
         job,
@@ -517,14 +510,35 @@ async def logs(root: Root, job: str) -> None:
             JobStatus.FAILED,
         },
     )
-    await _print_logs(root, id)
+    await process_logs(root, id, None)
 
 
-async def _print_logs(root: Root, job: str) -> None:
-    async for chunk in root.client.jobs.monitor(job):
-        if not chunk:
-            break
-        click.echo(chunk.decode(errors="replace"), nl=False)
+@command()
+@argument("job", type=JOB)
+async def attach(root: Root, job: str) -> None:
+    """
+    Attach local standard input, output, and error streams to a running job.
+    """
+    id = await resolve_job(
+        job,
+        client=root.client,
+        status={
+            JobStatus.PENDING,
+            JobStatus.RUNNING,
+            JobStatus.SUCCEEDED,
+            JobStatus.FAILED,
+        },
+    )
+    status = await root.client.jobs.status(id)
+    progress = JobStartProgress.create(tty=root.tty, color=root.color, quiet=root.quiet)
+    while status.status == JobStatus.PENDING:
+        await asyncio.sleep(0.2)
+        status = await root.client.jobs.status(id)
+        progress.step(status)
+    tty = status.container.tty
+    _check_tty(root, tty)
+
+    await process_attach(root, status, tty=tty, logs=False)
 
 
 @command()
@@ -939,7 +953,7 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     is_flag=True,
     help="Don't attach to job logs and don't wait for exit code",
 )
-@option("-t", "--tty", is_flag=True, help="Allocate a TTY")
+@TTY_OPT
 async def run(
     root: Root,
     image: RemoteImage,
@@ -962,14 +976,14 @@ async def run(
     pass_config: bool,
     browse: bool,
     detach: bool,
-    tty: bool,
+    tty: Optional[bool],
 ) -> None:
     """
     Run a job with predefined resources configuration.
 
-    IMAGE container image name.
+    IMAGE docker image name to run in a job.
 
-    CMD list will be passed as commands to model container.
+    CMD list will be passed as arguments to the executed job's image.
 
     Examples:
 
@@ -991,6 +1005,8 @@ async def run(
             "-p/-P option is deprecated and ignored. Use corresponding presets instead."
         )
     log.info(f"Using preset '{preset}': {job_preset}")
+    if tty is None:
+        tty = root.tty
     await run_job(
         root,
         image=image,
@@ -1034,6 +1050,7 @@ job.add_command(kill)
 job.add_command(top)
 job.add_command(save)
 job.add_command(browse)
+job.add_command(attach)
 
 
 job.add_command(alias(ls, "list", hidden=True))
@@ -1083,6 +1100,8 @@ async def run_job(
         raise click.UsageError("Cannot use --browse and --no-wait-start together")
     if not wait_start:
         detach = True
+    if not detach:
+        _check_tty(root, tty)
 
     job_restart_policy = JobRestartPolicy(restart)
     log.debug(f"Job restart policy: {job_restart_policy}")
@@ -1148,39 +1167,23 @@ async def run_job(
         restart_policy=job_restart_policy,
         life_span=job_life_span,
     )
-    click.echo(JobFormatter(root.quiet)(job))
     progress = JobStartProgress.create(tty=root.tty, color=root.color, quiet=root.quiet)
+    progress.begin(job)
     while wait_start and job.status == JobStatus.PENDING:
         await asyncio.sleep(0.2)
         job = await root.client.jobs.status(job.id)
-        progress(job)
-    progress.close()
-    if browse and job.status != JobStatus.FAILED:
-        await browse_job(root, job)
-
+        progress.step(job)
+    progress.end(job)
     # Even if we detached, but the job has failed to start
     # (most common reason - no resources), the command fails
     if job.status == JobStatus.FAILED:
         sys.exit(job.history.exit_code or EX_PLATFORMERROR)
 
+    if browse:
+        await browse_job(root, job)
+
     if not detach:
-        if not root.quiet:
-            msg = textwrap.dedent(
-                """\
-                Terminal is attached to the remote job, so you receive the job's output.
-                Use 'Ctrl-C' to detach (it will NOT terminate the job), or restart the
-                job with `--detach` option.\
-                """
-            )
-            click.echo(click.style(msg, dim=True))
-        await _print_logs(root, job.id)
-        job = await root.client.jobs.status(job.id)
-        while job.status in (JobStatus.PENDING, JobStatus.RUNNING):
-            await asyncio.sleep(0.1)
-            job = await root.client.jobs.status(job.id)
-        if job.status == JobStatus.FAILED:
-            sys.exit(job.history.exit_code or EX_PLATFORMERROR)
-        sys.exit(job.history.exit_code)
+        await process_attach(root, job, tty=tty, logs=True)
 
     return job
 
@@ -1289,8 +1292,8 @@ async def upload_and_map_config(root: Root) -> Tuple[str, Volume]:
 async def browse_job(root: Root, job: JobDescription) -> None:
     url = job.http_url
     if url.scheme not in ("http", "https"):
-        raise RuntimeError(f"Cannot open job URL: {url}")
-    log.info(f"Open job URL: {url}")
+        raise RuntimeError(f"Cannot browse job URL: {url}")
+    click.echo(f"Browsing job, please open: {url}")
     loop = asyncio.get_event_loop()
     await loop.run_in_executor(None, webbrowser.open, str(url))
 
@@ -1391,3 +1394,11 @@ def _parse_date(value: str) -> Optional[datetime]:
             raise ValueError("Date should be in ISO-8601 format")
     else:
         return None
+
+
+def _check_tty(root: Root, tty: bool) -> None:
+    if tty and not root.tty:
+        raise RuntimeError(
+            "The operation should be executed from a terminal, "
+            "the input device is not a TTY"
+        )
