@@ -1,3 +1,4 @@
+import asyncio
 import json
 from typing import Any, Callable, Dict, List, Optional
 
@@ -1015,6 +1016,106 @@ async def test_job_run_no_volumes(
         assert ret == _job_description_from_api(JSON, client.parse)
 
 
+async def test_job_run_with_relative_volume_uris(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    JSON = {
+        "id": "job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
+        "status": "failed",
+        "history": {
+            "status": "failed",
+            "reason": "Error",
+            "description": "Mounted on Avail\\n/dev/shm     " "64M\\n\\nExit code: 1",
+            "created_at": "2018-09-25T12:28:21.298672+00:00",
+            "started_at": "2018-09-25T12:28:59.759433+00:00",
+            "finished_at": "2018-09-25T12:28:59.759433+00:00",
+        },
+        "owner": "owner",
+        "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
+        "container": {
+            "image": "gcr.io/light-reality-205619/ubuntu:latest",
+            "command": "date",
+            "resources": {
+                "cpu": 1.0,
+                "memory_mb": 16384,
+                "gpu": 1,
+                "shm": False,
+                "gpu_model": "nvidia-tesla-p4",
+            },
+        },
+        "http_url": "http://my_host:8889",
+        "ssh_server": "ssh://my_host.ssh:22",
+        "ssh_auth_server": "ssh://my_host.ssh:22",
+        "is_preemptible": False,
+    }
+
+    async def handler(request: web.Request) -> web.Response:
+        data = await request.json()
+        assert data == {
+            "container": {
+                "image": "submit-image-name",
+                "command": "submit-command",
+                "http": {"port": 8181, "requires_auth": True},
+                "resources": {
+                    "memory_mb": 16384,
+                    "cpu": 7.0,
+                    "shm": True,
+                    "gpu": 1,
+                    "gpu_model": "test-gpu-model",
+                },
+                "volumes": [
+                    {
+                        "src_storage_uri": "storage://default/user/path",
+                        "dst_path": "/container/my_path",
+                        "read_only": False,
+                    },
+                    {
+                        "src_storage_uri": "storage://default/otheruser/path",
+                        "dst_path": "/container/other_user_path",
+                        "read_only": True,
+                    },
+                    {
+                        "src_storage_uri": "storage://othercluster/otheruser/path",
+                        "dst_path": "/container/other_cluster_path",
+                        "read_only": True,
+                    },
+                ],
+            },
+            "is_preemptible": False,
+            "cluster_name": "default",
+        }
+
+        return web.json_response(JSON)
+
+    app = web.Application()
+    app.router.add_post("/jobs", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        resources = Resources(16384, 7, 1, "test-gpu-model", True, None, None)
+        volumes: List[Volume] = [
+            Volume(URL("storage:path"), "/container/my_path", False),
+            Volume(URL("storage:/otheruser/path"), "/container/other_user_path", True,),
+            Volume(
+                URL("storage://othercluster/otheruser/path"),
+                "/container/other_cluster_path",
+                True,
+            ),
+        ]
+        container = Container(
+            image=RemoteImage.new_external_image(name="submit-image-name"),
+            command="submit-command",
+            resources=resources,
+            volumes=volumes,
+            http=HTTPPort(8181),
+        )
+        ret = await client.jobs.run(container=container, is_preemptible=False)
+
+        assert ret == _job_description_from_api(JSON, client.parse)
+
+
 async def test_job_run_preemptible(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -1818,3 +1919,32 @@ async def test_job_run_restart_policy(
         await client.jobs.run(
             container=container, restart_policy=JobRestartPolicy.ALWAYS
         )
+
+
+async def test_port_forward(
+    aiohttp_server: _TestServerFactory,
+    make_client: _MakeClient,
+    aiohttp_unused_port: Callable[[], int],
+) -> None:
+
+    port = aiohttp_unused_port()
+
+    async def handler(request: web.Request) -> web.WebSocketResponse:
+        resp = web.WebSocketResponse()
+        await resp.prepare(request)
+        async for msg in resp:
+            await resp.send_bytes(b"rep-" + msg.data)
+        return resp
+
+    app = web.Application()
+    app.router.add_get("/jobs/job-id/port_forward/12345", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        async with client.jobs.port_forward("job-id", port, 12345):
+            reader, writer = await asyncio.open_connection("127.0.0.1", port)
+            for i in range(5):
+                writer.write(str(i).encode("ascii"))
+                ret = await reader.read(1024)
+                assert ret == b"rep-" + str(i).encode("ascii")
