@@ -939,8 +939,8 @@ async def test_storage_upload_recursive_ok(
         await client.storage.upload_dir(
             URL(DATA_FOLDER.as_uri()) / "nested", URL("storage:folder")
         )
-    diff = dircmp(DATA_FOLDER / "nested", target_dir)  # type: ignore
-    assert not calc_diff(diff)  # type: ignore
+    diff = dircmp(DATA_FOLDER / "nested", target_dir)
+    assert not calc_diff(diff)
 
 
 async def test_storage_upload_recursive_slash_ending(
@@ -953,8 +953,8 @@ async def test_storage_upload_recursive_slash_ending(
         await client.storage.upload_dir(
             URL(DATA_FOLDER.as_uri()) / "nested", URL("storage:folder/")
         )
-    diff = dircmp(DATA_FOLDER / "nested", target_dir)  # type: ignore
-    assert not calc_diff(diff)  # type: ignore
+    diff = dircmp(DATA_FOLDER / "nested", target_dir)
+    assert not calc_diff(diff)
 
 
 async def test_storage_download_regular_file_to_absent_file(
@@ -1084,8 +1084,8 @@ async def test_storage_download_dir(
             URL("storage:folder"), URL(target_dir.as_uri())
         )
 
-    diff = dircmp(DATA_FOLDER / "nested", target_dir)  # type: ignore
-    assert not calc_diff(diff)  # type: ignore
+    diff = dircmp(DATA_FOLDER / "nested", target_dir)
+    assert not calc_diff(diff)
 
 
 async def test_storage_download_dir_slash_ending(
@@ -1101,8 +1101,8 @@ async def test_storage_download_dir_slash_ending(
             URL("storage:folder"), URL(local_dir.as_uri() + "/")
         )
 
-    diff = dircmp(DATA_FOLDER / "nested", local_dir / "nested")  # type: ignore
-    assert not calc_diff(diff)  # type: ignore
+    diff = dircmp(DATA_FOLDER / "nested", local_dir / "nested")
+    assert not calc_diff(diff)
 
 
 @pytest.fixture
@@ -1244,3 +1244,28 @@ async def test_storage_download_dir_update(
             URL("storage:folder"), URL(local_dir.as_uri()), update=True
         )
     assert local_file.read_bytes() == b"xxx"
+
+
+async def test_storage_upload_dir_with_ignore_file_names(
+    storage_server: Any, make_client: _MakeClient, tmp_path: Path, storage_path: Path
+) -> None:
+    local_dir = tmp_path / "folder"
+    local_dir2 = local_dir / "nested"
+    local_dir2.mkdir(parents=True)
+    for name in "one", "two", "three":
+        (local_dir / name).write_bytes(b"")
+        (local_dir2 / name).write_bytes(b"")
+    (local_dir / ".neuroignore").write_text("one")
+    (local_dir2 / ".gitignore").write_text("two")
+
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_dir(
+            URL(local_dir.as_uri()),
+            URL("storage:folder"),
+            ignore_file_names={".neuroignore", ".gitignore"},
+        )
+
+    names = sorted(os.listdir(storage_path / "folder"))
+    assert names == [".neuroignore", "nested", "three", "two"]
+    names = sorted(os.listdir(storage_path / "folder" / "nested"))
+    assert names == [".gitignore", "three"]
