@@ -1,5 +1,4 @@
 import textwrap
-from sys import platform
 
 from neuromation.api.admin import (
     _CloudProvider,
@@ -53,14 +52,6 @@ class TestClustersFormatter:
             is_tpu_enabled=is_tpu_enabled,
         )
 
-    @property
-    def _yes(self) -> str:
-        return "Yes" if platform == "win32" else " ✔︎"
-
-    @property
-    def _no(self) -> str:
-        return "No" if platform == "win32" else "✖︎"
-
     def test_cluster_list(self) -> None:
         formatter = ClustersFormatter()
         clusters = [_Cluster(name="default", status="deployed")]
@@ -176,7 +167,7 @@ class TestClustersFormatter:
               \x1b[1mRegion: \x1b[0mus-central1
               \x1b[1mNode pools:\x1b[0m
                 Machine       CPU  Memory  Preemptible  GPU  TPU  Min  Max  Idle
-                n1-highmem-8  7.0   45.0G      {self._yes}           {self._yes}    1    2     1
-                n1-highmem-8  7.0   45.0G       {self._no}            {self._no}    1    2     0"""  # noqa: E501, ignore line length
+                n1-highmem-8  7.0   45.0G       √             √     1    2     1
+                n1-highmem-8  7.0   45.0G       ×             ×     1    2     0"""
         )
         assert "\n".join(formatter(clusters)) == expected_out
