@@ -16,11 +16,11 @@ from neuromation.api import (
     StorageProgressStart,
     StorageProgressStep,
 )
-from neuromation.cli.formatters import create_storage_progress
 from neuromation.cli.formatters.storage import (
     BaseStorageProgress,
     StreamProgress,
     TTYProgress,
+    create_storage_progress,
     format_url,
 )
 from neuromation.cli.root import Root
@@ -67,6 +67,7 @@ def make_root() -> Iterator[_MakeRoot]:
             command_path="",
             command_params=[],
             skip_gmp_stats=True,
+            show_traceback=False,
         )
         return root
 
