@@ -1,8 +1,7 @@
 from pathlib import Path
-from typing import Any, Callable, Dict, NoReturn, Tuple
+from typing import Any, Callable, Dict, NoReturn
 from unittest import mock
 
-import click
 import pytest
 from aiohttp import web
 from yarl import URL
@@ -10,7 +9,6 @@ from yarl import URL
 from neuromation.api import Action, Client, JobStatus
 from neuromation.cli.root import Root
 from neuromation.cli.utils import (
-    LocalRemotePortParamType,
     pager_maybe,
     parse_file_resource,
     parse_permission_action,
@@ -28,6 +26,7 @@ def _job_entry(job_id: str) -> Dict[str, Any]:
         "id": job_id,
         "owner": "job-owner",
         "cluster_name": "default",
+        "uri": f"job://default/job-owner/{job_id}",
         "status": "running",
         "history": {
             "status": "running",
@@ -60,13 +59,19 @@ async def test_resolve_job_id__from_string__no_jobs_found(
         # Since `resolve_job` excepts any Exception, `assert` will be caught there
         name = request.query.get("name")
         if name != job_id:
-            raise web.HTTPBadRequest(text=(f"received: {name}"))
+            raise web.HTTPBadRequest(text=(f"received: name={name}"))
         owner = request.query.get("owner")
         if owner != "user":
-            raise web.HTTPBadRequest(text=(f"received: {owner}"))
+            raise web.HTTPBadRequest(text=(f"received: owner={owner}"))
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            raise web.HTTPBadRequest(text=(f"received: reverse={reverse}"))
+        limit = request.query.get("limit")
+        if limit != "1":
+            raise web.HTTPBadRequest(text=(f"received: limit={limit}"))
         status = request.query.getall("status")
         if status != ["running"]:
-            raise web.HTTPBadRequest(text=(f"received: {status}"))
+            raise web.HTTPBadRequest(text=(f"received: status={status}"))
         return web.json_response(JSON)
 
     app = web.Application()
@@ -95,6 +100,12 @@ async def test_resolve_job_id__from_uri_with_owner__no_jobs_found(
         owner = request.query.get("owner")
         if owner != job_owner:
             pytest.fail(f"received: {owner}")
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            pytest.fail(f"received: reverse={reverse}")
+        limit = request.query.get("limit")
+        if limit != "1":
+            pytest.fail(f"received: limit={limit}")
         return web.json_response(JSON)
 
     app = web.Application()
@@ -122,6 +133,12 @@ async def test_resolve_job_id__from_uri_without_owner__no_jobs_found(
         owner = request.query.get("owner")
         if owner != "user":
             pytest.fail(f"received: {owner}")
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            pytest.fail(f"received: reverse={reverse}")
+        limit = request.query.get("limit")
+        if limit != "1":
+            pytest.fail(f"received: limit={limit}")
         return web.json_response(JSON)
 
     app = web.Application()
@@ -149,6 +166,12 @@ async def test_resolve_job_id__from_string__single_job_found(
         owner = request.query.get("owner")
         if owner != "user":
             pytest.fail(f"received: {owner}")
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            pytest.fail(f"received: reverse={reverse}")
+        limit = request.query.get("limit")
+        if limit != "1":
+            pytest.fail(f"received: limit={limit}")
         return web.json_response(JSON)
 
     app = web.Application()
@@ -180,6 +203,12 @@ async def test_resolve_job_id__from_uri_with_owner__single_job_found(
         owner = request.query.get("owner")
         if owner != job_owner:
             pytest.fail(f"received: {owner}")
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            pytest.fail(f"received: reverse={reverse}")
+        limit = request.query.get("limit")
+        if limit != "1":
+            pytest.fail(f"received: limit={limit}")
         return web.json_response(JSON)
 
     app = web.Application()
@@ -208,6 +237,12 @@ async def test_resolve_job_id__from_uri_without_owner__single_job_found(
         owner = request.query.get("owner")
         if owner != "user":
             pytest.fail(f"received: {owner}")
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            pytest.fail(f"received: reverse={reverse}")
+        limit = request.query.get("limit")
+        if limit != "1":
+            pytest.fail(f"received: limit={limit}")
         return web.json_response(JSON)
 
     app = web.Application()
@@ -220,95 +255,6 @@ async def test_resolve_job_id__from_uri_without_owner__single_job_found(
         assert resolved == job_id
 
 
-async def test_resolve_job_id__from_string__multiple_jobs_found(
-    aiohttp_server: _TestServerFactory, make_client: _MakeClient
-) -> None:
-    job_name = "job-name-123-000"
-    job_id_1 = "job-id-1"
-    job_id_2 = "job-id-2"
-    JSON = {"jobs": [_job_entry(job_id_1), _job_entry(job_id_2)]}
-
-    async def handler(request: web.Request) -> web.Response:
-        # Since `resolve_job` excepts any Exception, `assert` will be caught there
-        name = request.query.get("name")
-        if name != job_name:
-            pytest.fail(f"received: {name}")
-        owner = request.query.get("owner")
-        if owner != "user":
-            pytest.fail(f"received: {owner}")
-        return web.json_response(JSON)
-
-    app = web.Application()
-    app.router.add_get("/jobs", handler)
-
-    srv = await aiohttp_server(app)
-
-    async with make_client(srv.make_url("/")) as client:
-        resolved = await resolve_job(
-            job_name, client=client, status={JobStatus.RUNNING}
-        )
-        assert resolved == job_id_2
-
-
-async def test_resolve_job_id__from_uri_with_owner__multiple_jobs_found(
-    aiohttp_server: _TestServerFactory, make_client: _MakeClient
-) -> None:
-    job_owner = "job-owner"
-    job_name = "job-name"
-    uri = f"job://default/{job_owner}/{job_name}"
-    job_id_1 = "job-id-1"
-    job_id_2 = "job-id-2"
-    JSON = {"jobs": [_job_entry(job_id_1), _job_entry(job_id_2)]}
-
-    async def handler(request: web.Request) -> web.Response:
-        # Since `resolve_job` excepts any Exception, `assert` will be caught there
-        name = request.query.get("name")
-        if name != job_name:
-            pytest.fail(f"received: {name}")
-        owner = request.query.get("owner")
-        if owner != job_owner:
-            pytest.fail(f"received: {owner}")
-        return web.json_response(JSON)
-
-    app = web.Application()
-    app.router.add_get("/jobs", handler)
-
-    srv = await aiohttp_server(app)
-
-    async with make_client(srv.make_url("/")) as client:
-        resolved = await resolve_job(uri, client=client, status={JobStatus.RUNNING})
-        assert resolved == job_id_2
-
-
-async def test_resolve_job_id__from_uri_without_owner__multiple_jobs_found(
-    aiohttp_server: _TestServerFactory, make_client: _MakeClient
-) -> None:
-    job_name = "job-name"
-    uri = f"job:{job_name}"
-    job_id_1 = "job-id-1"
-    job_id_2 = "job-id-2"
-    JSON = {"jobs": [_job_entry(job_id_1), _job_entry(job_id_2)]}
-
-    async def handler(request: web.Request) -> web.Response:
-        # Since `resolve_job` excepts any Exception, `assert` will be caught there
-        name = request.query.get("name")
-        if name != job_name:
-            pytest.fail(f"received: {name}")
-        owner = request.query.get("owner")
-        if owner != "user":
-            pytest.fail(f"received: {owner}")
-        return web.json_response(JSON)
-
-    app = web.Application()
-    app.router.add_get("/jobs", handler)
-
-    srv = await aiohttp_server(app)
-
-    async with make_client(srv.make_url("/")) as client:
-        resolved = await resolve_job(uri, client=client, status={JobStatus.RUNNING})
-        assert resolved == job_id_2
-
-
 async def test_resolve_job_id__server_error(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -323,6 +269,12 @@ async def test_resolve_job_id__server_error(
         owner = request.query.get("owner")
         if owner != "user":
             pytest.fail(f"received: {owner}")
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            pytest.fail(f"received: reverse={reverse}")
+        limit = request.query.get("limit")
+        if limit != "1":
+            pytest.fail(f"received: limit={limit}")
         raise web.HTTPError()
 
     app = web.Application()
@@ -352,6 +304,12 @@ async def test_resolve_job_id__from_uri_with_owner__with_owner__server_error(
         owner = request.query.get("owner")
         if owner != job_owner:
             pytest.fail(f"received: {owner}")
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            pytest.fail(f"received: reverse={reverse}")
+        limit = request.query.get("limit")
+        if limit != "1":
+            pytest.fail(f"received: limit={limit}")
         raise web.HTTPError()
 
     app = web.Application()
@@ -378,6 +336,12 @@ async def test_resolve_job_id__from_uri_without_owner__server_error(
         owner = request.query.get("owner")
         if owner != "user":
             pytest.fail(f"received: {owner}")
+        reverse = request.query.get("reverse")
+        if reverse != "1":
+            pytest.fail(f"received: reverse={reverse}")
+        limit = request.query.get("limit")
+        if limit != "1":
+            pytest.fail(f"received: limit={limit}")
         raise web.HTTPError()
 
     app = web.Application()
@@ -562,35 +526,6 @@ def test_parse_permission_action_wrong_empty() -> None:
         parse_permission_action(action)
 
 
-@pytest.mark.parametrize(
-    "arg,val",
-    [("1:1", (1, 1)), ("1:10", (1, 10)), ("434:1", (434, 1)), ("0897:123", (897, 123))],
-)
-def test_local_remote_port_param_type_valid(arg: str, val: Tuple[int, int]) -> None:
-    param = LocalRemotePortParamType()
-    assert param.convert(arg, None, None) == val
-
-
-@pytest.mark.parametrize(
-    "arg",
-    [
-        "1:",
-        "-123:10",
-        "34:-65500",
-        "hello:45",
-        "5555:world",
-        "65536:1",
-        "0:0",
-        "none",
-        "",
-    ],
-)
-def test_local_remote_port_param_type_invalid(arg: str) -> None:
-    param = LocalRemotePortParamType()
-    with pytest.raises(click.BadParameter, match=".* is not a valid port combination"):
-        param.convert(arg, None, None)
-
-
 def test_pager_maybe_no_tty() -> None:
     with mock.patch.multiple(
         "click", echo=mock.DEFAULT, echo_via_pager=mock.DEFAULT
