@@ -28,10 +28,12 @@ from uuid import uuid4 as uuid
 
 import aiodocker
 import aiohttp
+import pexpect
 import pytest
 from yarl import URL
 
 from neuromation.api import (
+    Action,
     AuthorizationError,
     Config,
     Container,
@@ -358,8 +360,7 @@ class Helper:
             job = await client.jobs.status(job_id)
             assert job.status == state
 
-    @run_async
-    async def job_info(self, job_id: str, wait_start: bool = False) -> JobDescription:
+    async def ajob_info(self, job_id: str, wait_start: bool = False) -> JobDescription:
         __tracebackhide__ = True
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
             job = await client.jobs.status(job_id)
@@ -374,6 +375,8 @@ class Helper:
                 raise AssertionError(f"timeout exceeded, last output: '{job.status}'")
             return job
 
+    job_info = run_async(ajob_info)
+
     @run_async
     async def check_job_output(
         self, job_id: str, expected: str, flags: int = 0
@@ -405,19 +408,8 @@ class Helper:
             f"Output of job {job_id} does not satisfy to expected regexp: {expected}"
         )
 
-    def run_cli(
-        self,
-        arguments: List[str],
-        *,
-        verbosity: int = 0,
-        network_timeout: float = NETWORK_TIMEOUT,
-    ) -> SysCap:
-        __tracebackhide__ = True
-
-        log.info("Run 'neuro %s'", " ".join(arguments))
-
+    def _default_args(self, verbosity: int, network_timeout: float) -> List[str]:
         args = [
-            "neuro",
             "--show-traceback",
             "--disable-pypi-version-check",
             "--color=no",
@@ -433,13 +425,29 @@ class Helper:
         if self._nmrc_path:
             args.append(f"--neuromation-config={self._nmrc_path}")
 
+        return args
+
+    def run_cli(
+        self,
+        arguments: List[str],
+        *,
+        verbosity: int = 0,
+        network_timeout: float = NETWORK_TIMEOUT,
+        input: Optional[str] = None,
+    ) -> SysCap:
+        __tracebackhide__ = True
+
+        log.info("Run 'neuro %s'", " ".join(arguments))
+
         # 5 min timeout is overkill
         proc = subprocess.run(
-            args + arguments,
+            ["neuro"] + self._default_args(verbosity, network_timeout) + arguments,
             timeout=300,
             encoding="utf8",
+            errors="replace",
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
+            input=input,
         )
         try:
             proc.check_returncode()
@@ -459,29 +467,41 @@ class Helper:
         out = out.strip()
         err = err.strip()
         if verbosity > 0:
-            print(f"nero stdout: {out}")
-            print(f"nero stderr: {err}")
+            print(f"neuro stdout: {out}")
+            print(f"neuro stderr: {err}")
         return SysCap(out, err)
 
+    def pexpect(
+        self,
+        arguments: List[str],
+        *,
+        verbosity: int = 0,
+        network_timeout: float = NETWORK_TIMEOUT,
+        encoding: str = "utf8",
+        echo: bool = True,
+    ) -> "pexpect.spawn":
+        if not hasattr(pexpect, "spawn"):
+            pytest.skip("TTY tests are not supported on Windows")
+        return pexpect.spawn(
+            "neuro",
+            self._default_args(verbosity, network_timeout) + arguments,
+            encoding=encoding,
+            echo=echo,
+        )
+
     def autocomplete(
-        self, arguments: List[str], *, network_timeout: float = NETWORK_TIMEOUT,
+        self,
+        arguments: List[str],
+        *,
+        verbosity: int = 0,
+        network_timeout: float = NETWORK_TIMEOUT,
+        timeout: float = 30,
     ) -> str:
         __tracebackhide__ = True
 
         log.info("Run 'neuro %s'", " ".join(arguments))
 
-        args = [
-            "neuro",
-            "--show-traceback",
-            "--disable-pypi-version-check",
-            "--color=no",
-            f"--network-timeout={network_timeout}",
-            "--skip-stats",
-        ]
-
-        if self._nmrc_path:
-            args.append(f"--neuromation-config={self._nmrc_path}")
-
+        args = self._default_args(verbosity, network_timeout)
         env = dict(os.environ)
         env["_NEURO_COMPLETE"] = "complete_zsh"
         env["COMP_WORDS"] = " ".join(shlex.quote(arg) for arg in args + arguments)
@@ -489,24 +509,24 @@ class Helper:
 
         proc = subprocess.run(
             "neuro",
-            timeout=300,
             encoding="utf8",
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
             env=env,
+            timeout=timeout,
         )
         assert proc.returncode == 1
         assert not proc.stderr
         return proc.stdout
 
-    @run_async
-    async def run_job_and_wait_state(
+    async def arun_job_and_wait_state(
         self,
         image: str,
         command: str = "",
         *,
         description: Optional[str] = None,
         name: Optional[str] = None,
+        tty: bool = False,
         wait_state: JobStatus = JobStatus.RUNNING,
         stop_state: JobStatus = JobStatus.FAILED,
     ) -> str:
@@ -518,6 +538,7 @@ class Helper:
                 image=client.parse.remote_image(image),
                 command=command,
                 resources=resources,
+                tty=tty,
             )
             job = await client.jobs.run(
                 container,
@@ -541,6 +562,8 @@ class Helper:
 
             return job.id
 
+    run_job_and_wait_state = run_async(arun_job_and_wait_state)
+
     @run_async
     async def check_http_get(self, url: Union[URL, str]) -> str:
         """
@@ -561,8 +584,7 @@ class Helper:
                     request_info=resp.request_info,
                 )
 
-    @run_async
-    async def kill_job(self, id_or_name: str, *, wait: bool = True) -> None:
+    async def akill_job(self, id_or_name: str, *, wait: bool = True) -> None:
         __tracebackhide__ = True
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
             id = await resolve_job(
@@ -576,20 +598,23 @@ class Helper:
                         if stat.status not in (JobStatus.PENDING, JobStatus.RUNNING):
                             break
 
-    @run_async
-    async def create_bucket(self, name: str) -> None:
+    kill_job = run_async(akill_job)
+
+    async def acreate_bucket(self, name: str) -> None:
         __tracebackhide__ = True
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
             await client.blob_storage.create_bucket(name)
 
-    @run_async
-    async def delete_bucket(self, name: str) -> None:
+    create_bucket = run_async(acreate_bucket)
+
+    async def adelete_bucket(self, name: str) -> None:
         __tracebackhide__ = True
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
             await client.blob_storage.delete_bucket(name)
 
-    @run_async
-    async def cleanup_bucket(self, bucket_name: str) -> None:
+    delete_bucket = run_async(adelete_bucket)
+
+    async def acleanup_bucket(self, bucket_name: str) -> None:
         __tracebackhide__ = True
         # Each test needs a clean bucket state and we can't delete bucket until it's
         # cleaned
@@ -606,6 +631,22 @@ class Helper:
                 tasks.append(client.blob_storage.delete_blob(bucket_name, key=blob.key))
             await asyncio.gather(*tasks)
 
+    cleanup_bucket = run_async(acleanup_bucket)
+
+    @run_async
+    async def drop_stale_buckets(self, bucket_prefix: str) -> None:
+        __tracebackhide__ = True
+        async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
+            buckets = await client.blob_storage.list_buckets()
+            for bucket in buckets:
+                if (
+                    bucket.name.startswith(bucket_prefix)
+                    and bucket.creation_time < time() - 3600 * 4
+                    and bucket.permission in (Action.WRITE, Action.MANAGE)
+                ):
+                    await self.acleanup_bucket(bucket.name)
+                    await self.adelete_bucket(bucket.name)
+
     @run_async
     async def upload_blob(self, bucket_name: str, key: str, file: Path) -> None:
         __tracebackhide__ = True
@@ -634,10 +675,24 @@ class Helper:
             assert self.hash_hex(tmp_path) == checksum, "checksum test failed for {url}"
 
 
+# Cache at the session level to reduce amount of relogins.
+# Frequent relogins returns 500 Internal Server Error too often.
+_nmrc_path_user = _nmrc_path_admin = None
+
+
 @pytest.fixture
 def nmrc_path(tmp_path_factory: Any, request: Any) -> Optional[Path]:
+    global _nmrc_path_user
+    global _nmrc_path_admin
     require_admin = request.keywords.get("require_admin", False)
-    return _get_nmrc_path(tmp_path_factory, require_admin)
+    if require_admin:
+        if _nmrc_path_admin is None:
+            _nmrc_path_admin = _get_nmrc_path(tmp_path_factory, True)
+        return _nmrc_path_admin
+    else:
+        if _nmrc_path_user is None:
+            _nmrc_path_user = _get_nmrc_path(tmp_path_factory, False)
+        return _nmrc_path_user
 
 
 def _get_nmrc_path(tmp_path_factory: Any, require_admin: bool) -> Optional[Path]:
@@ -709,12 +764,19 @@ def data2(static_path: Path) -> Tuple[str, str]:
     return generate_random_file(folder, FILE_SIZE_B // 3)
 
 
+@pytest.fixture(scope="session")
+def data3(static_path: Path) -> Tuple[str, str]:
+    folder = static_path / "data3"
+    folder.mkdir()
+    return generate_random_file(folder, FILE_SIZE_B // 5)
+
+
 @pytest.fixture(scope="session")
 def nested_data(static_path: Path) -> Tuple[str, str, str]:
     root_dir = static_path / "neested_data" / "nested"
     nested_dir = root_dir / "directory" / "for" / "test"
     nested_dir.mkdir(parents=True, exist_ok=True)
-    generated_file, hash = generate_random_file(nested_dir, FILE_SIZE_B)
+    generated_file, hash = generate_random_file(nested_dir, FILE_SIZE_B // 3)
     return generated_file, hash, str(root_dir)
 
 
@@ -723,11 +785,13 @@ def _tmp_bucket_create(
     tmp_path_factory: Any, request: Any
 ) -> Iterator[Tuple[str, Helper]]:
     tmp_path = tmp_path_factory.mktemp("tmp_bucket" + str(uuid()))
-    tmpbucketname = f"neuro_test_e2e_{uuid()}"
+    tmpbucketname = f"neuro-e2e-{uuid()}"
     nmrc_path = _get_nmrc_path(tmp_path_factory, require_admin=True)
 
     helper = Helper(nmrc_path, tmp_path)
+
     try:
+        helper.drop_stale_buckets("neuro-e2e-")
         helper.create_bucket(tmpbucketname)
     except AuthorizationError:
         pytest.skip("No permission to create bucket for user E2E_TOKEN")
