@@ -14,24 +14,39 @@ import aiodocker
 import aiohttp
 import pytest
 from aiohttp.test_utils import unused_port
+from pexpect.replwrap import REPLWrapper
 from yarl import URL
 
 from neuromation.api import Container, JobStatus, RemoteImage, Resources, get as api_get
 from neuromation.cli.asyncio_utils import run
 from tests.e2e.conftest import CLIENT_TIMEOUT, Helper
-from tests.e2e.utils import JOB_TINY_CONTAINER_PARAMS, JOB_TINY_CONTAINER_PRESET
+from tests.e2e.utils import JOB_TINY_CONTAINER_PARAMS
 
 
 pytestmark = pytest.mark.e2e_job
 
+SKIP_NON_LINUX = pytest.mark.skipif(
+    sys.platform != "linux", reason="PTY tests require Linux box"
+)
+
 ALPINE_IMAGE_NAME = "alpine:latest"
 UBUNTU_IMAGE_NAME = "ubuntu:latest"
 NGINX_IMAGE_NAME = "nginx:latest"
-TEST_IMAGE_NAME = "neuro-cli-test"
 MIN_PORT = 49152
 MAX_PORT = 65535
 EXEC_TIMEOUT = 180
 
+ANSI_RE = re.compile(r"\033\[[;?0-9]*[a-zA-Z]")
+OSC_RE = re.compile(r"\x1b]0;.+\x07")
+
+
+def strip_ansi(s: str) -> str:
+    s1 = ANSI_RE.sub("", s)
+    s2 = OSC_RE.sub("", s1)
+    s3 = s2.replace("\x1b[K", "")
+    s4 = s3.replace("\x1b[!p", "")
+    return s4
+
 
 @pytest.mark.e2e
 def test_job_submit(helper: Helper) -> None:
@@ -76,13 +91,12 @@ def test_job_submit(helper: Helper) -> None:
             "sleep 10m; false",
         ]
     )
-    match = re.match("Job ID: (.+) Status:", captured.out)
+    match = re.match("Job ID: (.+)", captured.out)
     assert match is not None
     job_id = match.group(1)
     assert job_id.startswith("job-")
     assert job_id not in jobs_orig
     assert f"Name: {job_name}" in captured.out
-    assert re.search("Http URL: http", captured.out), captured.out
 
     # Check it is in a running,pending job list now
     captured = helper.run_cli(
@@ -130,7 +144,7 @@ def test_job_description(helper: Helper) -> None:
             command,
         ]
     )
-    match = re.match("Job ID: (.+) Status:", captured.out)
+    match = re.match("Job ID: (.+)", captured.out)
     assert match is not None
     job_id = match.group(1)
 
@@ -173,18 +187,9 @@ def test_job_tags(helper: Helper) -> None:
 
     command = "sleep 10m"
     captured = helper.run_cli(
-        [
-            "job",
-            "run",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
-            *tag_options,
-            "--no-wait-start",
-            UBUNTU_IMAGE_NAME,
-            command,
-        ]
+        ["job", "run", *tag_options, "--no-wait-start", UBUNTU_IMAGE_NAME, command]
     )
-    match = re.match("Job ID: (.+) Status:", captured.out)
+    match = re.match("Job ID: (.+)", captured.out)
     assert match is not None
     job_id = match.group(1)
 
@@ -201,17 +206,9 @@ def test_job_tags(helper: Helper) -> None:
 @pytest.mark.e2e
 def test_job_filter_by_date_range(helper: Helper) -> None:
     captured = helper.run_cli(
-        [
-            "job",
-            "run",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
-            "--no-wait-start",
-            UBUNTU_IMAGE_NAME,
-            "sleep 300",
-        ]
+        ["job", "run", "--no-wait-start", UBUNTU_IMAGE_NAME, "sleep 300"]
     )
-    match = re.match("Job ID: (.+) Status:", captured.out)
+    match = re.match("Job ID: (.+)", captured.out)
     assert match is not None
     job_id = match.group(1)
     now = datetime.now()
@@ -269,7 +266,7 @@ def test_e2e_no_env(helper: Helper) -> None:
     )
 
     out = captured.out
-    match = re.match("Job ID: (.+) Status:", out)
+    match = re.match("Job ID: (.+)", out)
     assert match is not None
     job_id = match.group(1)
 
@@ -298,7 +295,7 @@ def test_e2e_env(helper: Helper) -> None:
     )
 
     out = captured.out
-    match = re.match("Job ID: (.+) Status:", out)
+    match = re.match("Job ID: (.+)", out)
     assert match is not None
     job_id = match.group(1)
 
@@ -328,7 +325,7 @@ def test_e2e_env_from_local(helper: Helper) -> None:
     )
 
     out = captured.out
-    match = re.match("Job ID: (.+) Status:", out)
+    match = re.match("Job ID: (.+)", out)
     assert match is not None
     job_id = match.group(1)
 
@@ -359,7 +356,7 @@ def test_e2e_multiple_env(helper: Helper) -> None:
     )
 
     out = captured.out
-    match = re.match("Job ID: (.+) Status:", out)
+    match = re.match("Job ID: (.+)", out)
     assert match is not None
     job_id = match.group(1)
 
@@ -410,6 +407,7 @@ def test_e2e_ssh_exec_true(helper: Helper) -> None:
 
     captured = helper.run_cli(
         [
+            "--quiet",
             "job",
             "exec",
             "--no-tty",
@@ -420,10 +418,11 @@ def test_e2e_ssh_exec_true(helper: Helper) -> None:
             # use unrolled notation to check shlex.join()
             "bash",
             "-c",
-            "echo ok; true",
+            "true",
         ]
     )
-    assert captured.out == "ok"
+    assert captured.err == ""
+    assert captured.out == ""
     helper.kill_job(job_id, wait=False)
 
 
@@ -477,6 +476,7 @@ def test_e2e_ssh_exec_echo(helper: Helper) -> None:
 
     captured = helper.run_cli(
         [
+            "--quiet",
             "job",
             "exec",
             "--no-tty",
@@ -484,42 +484,23 @@ def test_e2e_ssh_exec_echo(helper: Helper) -> None:
             "--timeout",
             str(EXEC_TIMEOUT),
             job_id,
-            "echo 1",
+            'bash -c "sleep 15; echo ok"',
         ]
     )
-    assert captured.out == "1"
-    helper.kill_job(job_id, wait=False)
-
-
-@pytest.mark.e2e
-def test_e2e_ssh_exec_no_tty(helper: Helper) -> None:
-    command = 'bash -c "sleep 15m; false"'
-    job_id = helper.run_job_and_wait_state(UBUNTU_IMAGE_NAME, command)
-
-    with pytest.raises(subprocess.CalledProcessError) as cm:
-        helper.run_cli(
-            [
-                "job",
-                "exec",
-                "--no-tty",
-                "--no-key-check",
-                "--timeout",
-                str(EXEC_TIMEOUT),
-                job_id,
-                "[ -t 1 ]",
-            ]
-        )
-    assert cm.value.returncode == 1
+    assert captured.err == ""
+    assert captured.out == "ok"
     helper.kill_job(job_id, wait=False)
 
 
 @pytest.mark.e2e
+@SKIP_NON_LINUX
 def test_e2e_ssh_exec_tty(helper: Helper) -> None:
     command = 'bash -c "sleep 15m; false"'
     job_id = helper.run_job_and_wait_state(UBUNTU_IMAGE_NAME, command)
 
-    captured = helper.run_cli(
+    expect = helper.pexpect(
         [
+            "--quiet",
             "job",
             "exec",
             "--no-key-check",
@@ -529,7 +510,7 @@ def test_e2e_ssh_exec_tty(helper: Helper) -> None:
             "[ -t 1 ]",
         ]
     )
-    assert captured.out == ""
+    assert expect.wait() == 0
     helper.kill_job(job_id, wait=False)
 
 
@@ -548,7 +529,7 @@ def test_e2e_ssh_exec_no_job(helper: Helper) -> None:
                 "true",
             ]
         )
-    assert cm.value.returncode == 127
+    assert cm.value.returncode == 65
 
 
 @pytest.mark.e2e
@@ -571,7 +552,7 @@ def test_e2e_ssh_exec_dead_job(helper: Helper) -> None:
                 "true",
             ]
         )
-    assert cm.value.returncode == 127
+    assert cm.value.returncode == 65
 
 
 @pytest.mark.e2e
@@ -734,16 +715,7 @@ def test_job_run(helper: Helper) -> None:
     # Run a new job
     command = 'bash -c "exit 101"'
     captured = helper.run_cli(
-        [
-            "-q",
-            "job",
-            "run",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
-            "--no-wait-start",
-            UBUNTU_IMAGE_NAME,
-            command,
-        ]
+        ["-q", "job", "run", "--no-wait-start", UBUNTU_IMAGE_NAME, command]
     )
     job_id = captured.out
 
@@ -763,8 +735,6 @@ def test_pass_config(helper: Helper) -> None:
             "-q",
             "job",
             "run",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
             "--no-wait-start",
             "--pass-config",
             UBUNTU_IMAGE_NAME,
@@ -807,86 +777,21 @@ def fakebrowser(monkeypatch: Any) -> None:
 def test_job_browse(helper: Helper, fakebrowser: Any) -> None:
     # Run a new job
     captured = helper.run_cli(
-        [
-            "-q",
-            "job",
-            "run",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
-            "--detach",
-            UBUNTU_IMAGE_NAME,
-            "true",
-        ]
+        ["-q", "job", "run", "--detach", UBUNTU_IMAGE_NAME, "true"]
     )
     job_id = captured.out
 
     captured = helper.run_cli(["-v", "job", "browse", job_id])
-    assert "Browsing https://job-" in captured.out
-    assert "Open job URL: https://job-" in captured.err
+    assert "Browsing job, please open: https://job-" in captured.out
 
 
 @pytest.mark.e2e
 def test_job_run_browse(helper: Helper, fakebrowser: Any) -> None:
     # Run a new job
     captured = helper.run_cli(
-        [
-            "-v",
-            "job",
-            "run",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
-            "--detach",
-            "--browse",
-            UBUNTU_IMAGE_NAME,
-            "true",
-        ]
+        ["-v", "job", "run", "--detach", "--browse", UBUNTU_IMAGE_NAME, "true"]
     )
-    assert "Browsing https://job-" in captured.out
-    assert "Open job URL: https://job-" in captured.err
-
-
-@pytest.mark.e2e
-def test_job_run_no_detach(helper: Helper) -> None:
-    token = uuid4()
-    # Run a new job
-    captured = helper.run_cli(
-        [
-            "-v",
-            "job",
-            "run",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
-            UBUNTU_IMAGE_NAME,
-            f"echo {token}",
-        ]
-    )
-    assert str(token) in captured.out
-    detach_notification = """\
-Terminal is attached to the remote job, so you receive the job's output.
-Use 'Ctrl-C' to detach (it will NOT terminate the job), or restart the job
-with `--detach` option.\
-"""
-    assert detach_notification
-
-
-@pytest.mark.e2e
-def test_job_run_no_detach_quiet_mode(helper: Helper) -> None:
-    token = str(uuid4())
-    # Run a new job
-    captured = helper.run_cli(
-        [
-            "-q",
-            "job",
-            "run",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
-            UBUNTU_IMAGE_NAME,
-            f"echo {token}",
-        ]
-    )
-    out = captured.out.strip()
-    assert "Use 'Ctrl-C' to detach (it will NOT terminate the job)" not in out
-    assert out.endswith(token)
+    assert "Browsing job, please open: https://job-" in captured.out
 
 
 @pytest.mark.e2e
@@ -918,8 +823,6 @@ def test_job_run_no_detach_browse_failure(helper: Helper) -> None:
                 "-v",
                 "job",
                 "run",
-                "-s",
-                JOB_TINY_CONTAINER_PRESET,
                 "--detach",
                 "--browse",
                 UBUNTU_IMAGE_NAME,
@@ -932,24 +835,10 @@ def test_job_run_no_detach_browse_failure(helper: Helper) -> None:
 
 @pytest.mark.e2e
 def test_job_run_with_tty(helper: Helper) -> None:
-    # Run a new job
     command = "test -t 0"
-    captured = helper.run_cli(
-        [
-            "-q",
-            "job",
-            "run",
-            "-t",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
-            UBUNTU_IMAGE_NAME,
-            command,
-        ]
+    job_id = helper.run_job_and_wait_state(
+        UBUNTU_IMAGE_NAME, command, wait_state=JobStatus.SUCCEEDED, tty=True
     )
-    job_id = captured.out
-
-    # Wait until the job is running
-    helper.wait_job_change_state_to(job_id, JobStatus.SUCCEEDED)
 
     captured = helper.run_cli(["job", "status", job_id])
     assert "TTY: True" in captured.out
@@ -961,24 +850,13 @@ def test_job_run_home_volumes_automount(helper: Helper, fakebrowser: Any) -> Non
 
     with pytest.raises(subprocess.CalledProcessError) as cm:
         # first, run without --volume=HOME
-        helper.run_cli(
-            ["-q", "job", "run", "--preset=cpu-micro", UBUNTU_IMAGE_NAME, command]
-        )
+        helper.run_cli(["-q", "job", "run", UBUNTU_IMAGE_NAME, command])
 
     assert cm.value.returncode == 1
 
     # then, run with --volume=HOME
     capture = helper.run_cli(
-        [
-            "-q",
-            "job",
-            "run",
-            "--preset=cpu-micro",
-            "--volume",
-            "HOME",
-            UBUNTU_IMAGE_NAME,
-            command,
-        ]
+        ["-q", "job", "run", "--volume", "HOME", UBUNTU_IMAGE_NAME, command]
     )
 
     job_id_2 = capture.out
@@ -992,7 +870,6 @@ def test_job_run_volume_all(helper: Helper) -> None:
         [
             f"[ -d {root_mountpoint}/{helper.username} ]",
             f"[ -d {root_mountpoint}/neuromation ]",  # must be public
-            f"[ -d {root_mountpoint}/test2/public ]",  # must be public
             f"[ $NEUROMATION_ROOT == {root_mountpoint} ]",
             f"[ $NEUROMATION_HOME == {root_mountpoint}/{helper.username} ]",
         ]
@@ -1002,18 +879,18 @@ def test_job_run_volume_all(helper: Helper) -> None:
 
     with pytest.raises(subprocess.CalledProcessError) as cm:
         # first, run without --volume=ALL
-        captured = helper.run_cli(["--quiet", "run", "-s", "cpu-micro", img, command])
+        captured = helper.run_cli(["--quiet", "run", "-T", img, command])
     assert cm.value.returncode == 1
 
     # then, run with --volume=ALL
-    captured = helper.run_cli(["run", "-s", "cpu-micro", "--volume=ALL", img, command])
+    captured = helper.run_cli(["run", "-T", "--volume=ALL", img, command])
     msg = (
         "Storage mountpoints will be available as the environment variables:\n"
         f"  NEUROMATION_ROOT={root_mountpoint}\n"
         f"  NEUROMATION_HOME={root_mountpoint}/{helper.username}"
     )
     assert msg in captured.out
-    found_job_ids = re.findall("Job ID: (job-.+) Status:", captured.out)
+    found_job_ids = re.findall("Job ID: (job-.+)", captured.out)
     assert len(found_job_ids) == 1
     job_id = found_job_ids[0]
     helper.wait_job_change_state_to(
@@ -1127,8 +1004,6 @@ def test_e2e_restart_failing(request: Any, helper: Helper) -> None:
             "run",
             "--restart",
             "on-failure",
-            "-s",
-            JOB_TINY_CONTAINER_PRESET,
             "--detach",
             UBUNTU_IMAGE_NAME,
             "false",
@@ -1178,3 +1053,54 @@ def test_job_autocomplete(helper: Helper) -> None:
     assert job_id in out
 
     helper.kill_job(job_id)
+
+
+@pytest.mark.e2e
+def test_job_run_stdout(helper: Helper) -> None:
+    command = 'bash -c "sleep 30; for count in {0..3}; do echo $count; sleep 1; done"'
+
+    try:
+        captured = helper.run_cli(
+            ["-q", "job", "run", "--no-tty", UBUNTU_IMAGE_NAME, command]
+        )
+    except subprocess.CalledProcessError as exc:
+        # EX_IOERR is returned if the process is not finished in 10 secs after
+        # disconnecting the attached session
+        assert exc.returncode == 74
+        err = exc.stderr
+        out = exc.stdout
+    else:
+        err = captured.err
+        out = captured.out
+
+    assert err == ""
+    assert "\n".join(f"{i}" for i in range(4)) in out
+
+
+@pytest.mark.e2e
+def test_job_attach_tty(helper: Helper) -> None:
+    job_id = helper.run_job_and_wait_state(UBUNTU_IMAGE_NAME, "sh", tty=True)
+
+    status = helper.job_info(job_id)
+    assert status.container.tty
+
+    expect = helper.pexpect(["job", "attach", job_id])
+    expect.expect("========== Job is running in terminal mode =========")
+    expect.sendline("")  # prompt may be missing after the connection.
+    repl = REPLWrapper(expect, "# ", None)
+    ret = repl.run_command("echo abc")
+    assert ret.strip() == "echo abc\r\r\nabc"
+
+    helper.kill_job(job_id)
+
+
+# The test doesn't work yet
+# @pytest.mark.e2e
+# def test_job_run_non_tty_stdin(helper: Helper) -> None:
+#     command = "wc --chars"
+#     captured = helper.run_cli(
+#         ["-q", "job", "run", UBUNTU_IMAGE_NAME, command], input="abcdef"
+#     )
+
+#     assert captured.err == ""
+#     assert captured.out == "6"
